%bcond_with gui_and_doc

Summary:       An extensible, customizable, free/libre text editor — and more.
Name:          emacs
Epoch:         1
Version:       29.1
Release:       6%{?dist}
License:       GPLv3+ and CC0
URL:           http://www.gnu.org/software/emacs/
Source0:       https://ftp.gnu.org/gnu/emacs/emacs-%{version}.tar.xz
Source2:       macros.emacs
# original source3 has already exits in source tree, store emacs.pc instead.
Source3:       emacs.pc
Source4:       dotemacs.el
Source5:       site-start.el
Source6:       default.el
Source7:       emacs-terminal.desktop
Source8:       emacs-terminal.sh
Source10:      %{name}.appdata.xml

# https://git.savannah.gnu.org/cgit/emacs.git/commit/?id=216c65b135c2b0be7e048cdc6683873b03b99b9a
Patch0001:     0001-emacs-libdir-vs-systemd.patch
# CVE-2024-30203 https://git.savannah.gnu.org/cgit/emacs.git/commit/?h=emacs-29&id=937b9042ad7426acdcca33e3d931d8f495bdd804
Patch0002:     0002-lisp-gnus-mm-view.el-mm-display-inline-fontify-Mark-.patch
# CVE-2024-30205 https://git.savannah.gnu.org/cgit/emacs.git/commit/?h=emacs-29&id=2bc865ace050ff118db43f01457f95f95112b877
Patch0003:     0003-org-file-contents-Consider-all-remote-files-unsafe.patch
# CVE-2024-30204 https://git.savannah.gnu.org/cgit/emacs.git/commit/?h=emacs-29&id=6f9ea396f49cbe38c2173e0a72ba6af3e03b271c
Patch0004:     0004-org-latex-preview-Add-protection-when-untrusted-cont.patch
# CVE-2024-30202 https://git.savannah.gnu.org/cgit/emacs.git/commit/?h=emacs-29&id=befa9fcaae29a6c9a283ba371c3c5234c7f644eb
Patch0005:     0005-org-macro-set-templates-Prevent-code-evaluation.patch
# CVE-2024-39331 https://git.savannah.gnu.org/cgit/emacs.git/commit/?id=c645e1d8205f0f0663ec4a2d27575b238c646c7c
Patch0006:     0006-org-link-expand-abbrev-Do-not-evaluate-arbitrary-uns.patch

# enforcing system crypto policies
Patch3000:     emacs-system-crypto-policies.patch


# building tools
BuildRequires: gcc, autoconf, make
# library
BuildRequires: glibc-devel, gnutls-devel, libselinux-devel, systemd-devel, gnupg2, libacl-devel, jansson-devel, bzip2, gzip, ncurses-devel

%if %{with gui_and_doc}
# gui
BuildRequires: Xaw3d-devel, libXpm-devel, libpng-devel, zlib-devel, libjpeg-turbo-devel, libjpeg-turbo, libtiff-devel, giflib-devel, librsvg2-devel, libwebp-devel, libX11-devel, atk-devel, cairo-devel, freetype-devel, fontconfig-devel, libXau-devel, libXdmcp-devel, libXrender-devel, libXt-devel, xorg-x11-proto-devel, gpm-devel, desktop-file-utils, liblockfile-devel, libxml2-devel, cairo, libappstream-glib, gtk3-devel, webkit2gtk3-devel, dbus-devel
# sound
BuildRequires: alsa-lib-devel
# document tools
BuildRequires: texinfo
# alternative library: m17n-lib-devel, libotf-devel
BuildRequires: harfbuzz-devel
%endif


Requires:      info
%if %{with gui_and_doc}
Requires:      desktop-file-utils
Requires:      dejavu-sans-mono-fonts
Requires(preun): %{_sbindir}/alternatives
Requires(posttrans): %{_sbindir}/alternatives
%endif
Requires:      emacs-common = %{epoch}:%{version}-%{release}
Provides:      emacs(bin) = %{epoch}:%{version}-%{release}


%define site_lisp %{_datadir}/emacs/site-lisp
%define site_start_d %{site_lisp}/site-start.d
%define bytecompargs -batch --no-init-file --no-site-file -f batch-byte-compile
%define pkgconfig %{_datadir}/pkgconfig
%define emacs_libexecdir %{_libexecdir}/emacs/%{version}/%{_host}

%description
Emacs is a powerful, customizable, self-documenting, modeless text
editor. Emacs contains special code editing features, a scripting
language (elisp), and the capability to read mail, news, and more
without leaving the editor.

This package provides an emacs binary with support for X windows.

# In TencentOS, Emacs will not build lucid and terminal package.
%if %{with gui_and_doc}
%package lucid
Summary:       GNU Emacs text editor with LUCID toolkit X support
Requires(preun): %{_sbindir}/alternatives
Requires(posttrans): %{_sbindir}/alternatives
Requires:      emacs-common = %{epoch}:%{version}-%{release}
Provides:      emacs(bin) = %{epoch}:%{version}-%{release}

%description lucid
Emacs is a powerful, customizable, self-documenting, modeless text
editor. Emacs contains special code editing features, a scripting
language (elisp), and the capability to read mail, news, and more
without leaving the editor.

This package provides an emacs binary with support for X windows
using LUCID toolkit.

%package nox
Summary:       GNU Emacs text editor without X support
Requires(preun): %{_sbindir}/alternatives
Requires(posttrans): %{_sbindir}/alternatives
Requires:      emacs-common = %{epoch}:%{version}-%{release}
Provides:      emacs(bin) = %{epoch}:%{version}-%{release}

%description nox
Emacs is a powerful, customizable, self-documenting, modeless text
editor. Emacs contains special code editing features, a scripting
language (elisp), and the capability to read mail, news, and more
without leaving the editor.

This package provides an emacs binary with no X windows support for running
on a terminal.

%package terminal
Summary:       A desktop menu item for GNU Emacs terminal.
Requires:      emacs = %{epoch}:%{version}-%{release}
BuildArch:     noarch

%description terminal
Contains a desktop menu item running GNU Emacs terminal. Install
emacs-terminal if you need a terminal with Malayalam support.

Please note that emacs-terminal is a temporary package and it will be
removed when another terminal becomes capable of handling Malayalam.
%endif

%package common
Summary:       Emacs common files
# The entire source code is GPLv3+ except lib-src/etags.c which is
# also BSD.  Manual (info) is GFDL.
License:       GPLv3+ and GFDL and BSD
%if %{with gui_and_doc}
Requires(preun): %{_sbindir}/alternatives
Requires(posttrans): %{_sbindir}/alternatives
%endif
Requires:      %{name}-filesystem = %{epoch}:%{version}-%{release}
Recommends:    enchant2
Provides:      %{name}-el = %{epoch}:%{version}-%{release}
Obsoletes:     emacs-el < 1:24.3-29

%description common
Emacs is a powerful, customizable, self-documenting, modeless text
editor. Emacs contains special code editing features, a scripting
language (elisp), and the capability to read mail, news, and more
without leaving the editor.

This package contains all the common files needed by emacs, emacs-lucid
or emacs-nox.

%package filesystem
Summary:       Emacs filesystem layout
BuildArch:     noarch

%description filesystem
This package provides some directories which are required by other
packages that add functionality to Emacs.

%package devel
Summary: Development header files for Emacs

%description devel
Development header files for Emacs.

%prep
%autosetup -p1

# pre-configure source
bash autogen.sh

# Avoid trademark issues
rm -f lisp/play/tetris.el lisp/play/tetris.elc
rm -f lisp/play/pong.el lisp/play/pong.el

# Avoid duplicating doc files in the common subpackage
ln -s ../../%{name}/%{version}/etc/COPYING doc
ln -s ../../%{name}/%{version}/etc/NEWS doc

%build
export CFLAGS="-DMAIL_USE_LOCKF %{build_cflags}"

# build GUI binary
%if %{with gui_and_doc}
# Build GTK+ binary
mkdir build-gtk && cd build-gtk
ln -s ../configure .
%configure --with-dbus --with-gif --with-jpeg --with-png --with-rsvg\
           --with-tiff --with-xft --with-xpm --with-x-toolkit=gtk3 --with-gpm=no \
           --with-xwidgets --with-modules --with-harfbuzz --with-cairo --with-json #mailutils-devel
%make_build
rm src/emacs-%{version}.*
cd ..

# Build Lucid binary
mkdir build-lucid && cd build-lucid
ln -s ../configure .
%configure --with-dbus --with-gif --with-jpeg --with-png --with-rsvg \
           --with-tiff --with-xft --with-xpm --with-x-toolkit=lucid --with-gpm=no \
           --with-modules --with-harfbuzz --with-cairo --with-json
%make_build
rm src/emacs-%{version}.*
cd ..
%endif

# Build binary without X support
mkdir build-nox && cd build-nox
ln -s ../configure .
%configure --with-x=no --with-modules --with-json
%make_build
rm src/emacs-%{version}.*
cd ..



%install
%if %{with gui_and_doc}
(cd build-gtk && %make_install)

# Let alternatives manage the symlink
rm %{buildroot}%{_bindir}/emacs
touch %{buildroot}%{_bindir}/emacs


# Install emacs.pdmp of the emacs with GTK+
install -p -m 0644 build-gtk/src/emacs.pdmp %{buildroot}%{_bindir}/emacs-%{version}.pdmp
# Install the emacs with LUCID toolkit
install -p -m 0755 build-lucid/src/emacs %{buildroot}%{_bindir}/emacs-%{version}-lucid
install -p -m 0644 build-lucid/src/emacs.pdmp %{buildroot}%{_bindir}/emacs-%{version}-lucid.pdmp
# Install the emacs without X
install -p -m 0755 build-nox/src/emacs %{buildroot}%{_bindir}/emacs-%{version}-nox
install -p -m 0644 build-nox/src/emacs.pdmp %{buildroot}%{_bindir}/emacs-%{version}-nox.pdmp

%else
(cd build-nox && %make_install)
# only emacs-nox exits, so copy it to bindir
rm %{buildroot}%{_bindir}/emacs

install -p -m 0755 build-nox/src/emacs %{buildroot}%{_bindir}/emacs-%{version}
(cd %{buildroot}%{_bindir} && ln -s emacs-%{version} emacs)
install -p -m 0644 build-nox/src/emacs.pdmp %{buildroot}%{_bindir}/emacs-%{version}.pdmp
(cd %{buildroot}%{_bindir} && ln -s emacs-%{version}.pdmp emacs.pdmp)
%endif

# Do not compress the files which implement compression itself (#484830)
gunzip %{buildroot}%{_datadir}/emacs/%{version}/lisp/jka-compr.el.gz
gunzip %{buildroot}%{_datadir}/emacs/%{version}/lisp/jka-cmpr-hook.el.gz

# Make sure movemail isn't setgid 
chmod 755 %{buildroot}%{emacs_libexecdir}/movemail

mkdir -p %{buildroot}%{site_lisp}
install -p -m 0644 %SOURCE5 %{buildroot}%{site_lisp}/site-start.el
install -p -m 0644 %SOURCE6 %{buildroot}%{site_lisp}

# This solves bz#474958, "update-directory-autoloads" now finally
# works the path is different each version, so we'll generate it here
echo "(setq source-directory \"%{_datadir}/emacs/%{version}/\")" \
 >> %{buildroot}%{site_lisp}/site-start.el

mv %{buildroot}%{_bindir}/etags %{buildroot}%{_bindir}/etags.emacs
mv %{buildroot}%{_mandir}/man1/ctags.1.gz %{buildroot}%{_mandir}/man1/gctags.1.gz
mv %{buildroot}%{_mandir}/man1/etags.1.gz %{buildroot}%{_mandir}/man1/etags.emacs.1.gz
mv %{buildroot}%{_bindir}/ctags %{buildroot}%{_bindir}/gctags
# BZ 927996
mv %{buildroot}%{_infodir}/info.info.gz %{buildroot}%{_infodir}/info.gz

mkdir -p %{buildroot}%{site_lisp}/site-start.d

# Default initialization file
mkdir -p %{buildroot}%{_sysconfdir}/skel
install -p -m 0644 %SOURCE4 %{buildroot}%{_sysconfdir}/skel/.emacs

# Install pkgconfig file
mkdir -p %{buildroot}/%{pkgconfig}
install -p -m 0644 %{SOURCE3} %{buildroot}/%{pkgconfig}
sed -i -e "s|@@epoch@@|%{epoch}|g" -e "s|@@version@@|%{version}|g" %{buildroot}/%{pkgconfig}/emacs.pc

# Install rpm macro definition file
mkdir -p %{buildroot}%{_rpmconfigdir}/macros.d
install -p -m 0644 %{SOURCE2} %{buildroot}%{_rpmconfigdir}/macros.d/
sed -i -e "s|@@epoch@@|%{epoch}|g" -e "s|@@version@@|%{version}|g" -e "s|@@release@@|%{release}|g" %{buildroot}%{_rpmconfigdir}/macros.d/macros.emacs

%if %{with gui_and_doc}
# Install app data
mkdir -p %{buildroot}/%{_datadir}/appdata
cp -a %SOURCE10 %{buildroot}/%{_datadir}/appdata
# Upstream ships its own appdata file, but it's quite terse.
rm %{buildroot}/%{_metainfodir}/emacs.appdata.xml
# Install desktop files
desktop-file-install --dir=%{buildroot}%{_datadir}/applications \
                     %SOURCE7                     
# Installing emacs-terminal binary
install -p -m 755 %SOURCE8 %{buildroot}%{_bindir}/emacs-terminal
%else
rm -rf %{buildroot}/%{_datadir}/{applications,icons,metainfo}
%endif

# After everything is installed, remove info dir
rm -f %{buildroot}%{_infodir}/dir

# Remove duplicate desktop-related files
rm %{buildroot}%{_datadir}/%{name}/%{version}/etc/%{name}.{desktop,service}

#
# Create file lists
#
( TOPDIR=${PWD}
  cd %{buildroot}

  find .%{_datadir}/emacs/%{version}/lisp \
    .%{_datadir}/emacs/%{version}/lisp/leim \
    .%{_datadir}/emacs/site-lisp \( -type f -name '*.elc' -fprint $TOPDIR/common-lisp-none-elc-files \) -o \( -type d -fprintf $TOPDIR/common-lisp-dir-files "%%%%dir %%p\n" \) -o \( -name '*.el.gz' -fprint $TOPDIR/el-bytecomped-files -o -fprint $TOPDIR/common-not-comped-files \)

)
# Put the lists together after filtering  ./usr to /usr
sed -i -e "s|\.%{_prefix}|%{_prefix}|" *-files
cat common-*-files > common-filelist
cat el-*-files common-lisp-dir-files > el-filelist

# Remove old icon
rm -f %{buildroot}%{_datadir}/icons/hicolor/scalable/mimetypes/emacs-document23.svg

%check
%if %{with gui_and_doc}
appstream-util validate-relax --nonet %{buildroot}%{_datadir}/appdata/*.appdata.xml
desktop-file-validate %{buildroot}/%{_datadir}/applications/*.desktop
%endif

%if %{with gui_and_doc}
%preun
%{_sbindir}/alternatives --remove emacs %{_bindir}/emacs-%{version} || :

%posttrans
%{_sbindir}/alternatives --install %{_bindir}/emacs emacs %{_bindir}/emacs-%{version} 80 || :

%preun lucid
%{_sbindir}/alternatives --remove emacs %{_bindir}/emacs-%{version}-lucid || :
%{_sbindir}/alternatives --remove emacs-lucid %{_bindir}/emacs-%{version}-lucid || :

%posttrans lucid
%{_sbindir}/alternatives --install %{_bindir}/emacs emacs %{_bindir}/emacs-%{version}-lucid 70 || :
%{_sbindir}/alternatives --install %{_bindir}/emacs-lucid emacs-lucid %{_bindir}/emacs-%{version}-lucid 60 || :

%preun nox
%{_sbindir}/alternatives --remove emacs %{_bindir}/emacs-%{version}-nox || :
%{_sbindir}/alternatives --remove emacs-nox %{_bindir}/emacs-%{version}-nox || :

%posttrans nox
%{_sbindir}/alternatives --install %{_bindir}/emacs emacs %{_bindir}/emacs-%{version}-nox 70 || :
%{_sbindir}/alternatives --install %{_bindir}/emacs-nox emacs-nox %{_bindir}/emacs-%{version}-nox 60 || :

%preun common
%{_sbindir}/alternatives --remove emacs.etags %{_bindir}/etags.emacs || :

%posttrans common
%{_sbindir}/alternatives --install %{_bindir}/etags emacs.etags %{_bindir}/etags.emacs 80 \
       --slave %{_mandir}/man1/etags.1.gz emacs.etags.man %{_mandir}/man1/etags.emacs.1.gz || :
%endif

%files
%{_bindir}/emacs-%{version}
%{_bindir}/emacs-%{version}.pdmp
%if %{with gui_and_doc}
%attr(0755,-,-) %ghost %{_bindir}/emacs
%{_datadir}/applications/emacs.desktop
%{_datadir}/appdata/%{name}.appdata.xml
%{_datadir}/icons/hicolor/*/apps/emacs.png
%{_datadir}/icons/hicolor/scalable/apps/emacs.svg
%{_datadir}/icons/hicolor/scalable/apps/emacs.ico
%{_datadir}/icons/hicolor/scalable/mimetypes/emacs-document.svg
%else
%{_bindir}/emacs
%{_bindir}/emacs.pdmp
%endif

%if %{with gui_and_doc}
%files lucid
%{_bindir}/emacs-%{version}-lucid
%{_bindir}/emacs-%{version}-lucid.pdmp
%attr(0755,-,-) %ghost %{_bindir}/emacs
%attr(0755,-,-) %ghost %{_bindir}/emacs-lucid

%files nox
%{_bindir}/emacs-%{version}-nox
%{_bindir}/emacs-%{version}-nox.pdmp
%attr(0755,-,-) %ghost %{_bindir}/emacs
%attr(0755,-,-) %ghost %{_bindir}/emacs-nox

%files terminal
%{_bindir}/emacs-terminal
%{_datadir}/applications/emacs-terminal.desktop
%endif

%files common -f common-filelist -f el-filelist
%config(noreplace) %{_sysconfdir}/skel/.emacs
%{_rpmconfigdir}/macros.d/macros.emacs
%license etc/COPYING
%doc doc/NEWS BUGS README
%{_bindir}/ebrowse
%{_bindir}/emacsclient
%{_bindir}/etags.emacs
%{_bindir}/gctags
%{_mandir}/*/*
%{_infodir}/*
%dir %{_datadir}/emacs/%{version}
%{_datadir}/emacs/%{version}/etc
%{_datadir}/emacs/%{version}/site-lisp
%{_libexecdir}/emacs
%{_userunitdir}/emacs.service
%attr(0644,root,root) %config(noreplace) %{_datadir}/emacs/site-lisp/default.el
%attr(0644,root,root) %config %{_datadir}/emacs/site-lisp/site-start.el
%{pkgconfig}/emacs.pc

%files filesystem
%dir %{_datadir}/emacs
%dir %{_datadir}/emacs/site-lisp
%dir %{_datadir}/emacs/site-lisp/site-start.d

%files devel
%{_includedir}/emacs-module.h

%changelog
* Fri Jul 19 2024 Yi Lin <nilusyi@tencent.com> - 1:29.1-6
- fix CVE-2024-30205

* Tue Jun 25 2024 Yi Lin <nilusyi@tencent.com> - 1:29.1-5
- fix CVE-2024-39331

* Mon Apr 08 2024 Yi Lin <nilusyi@tencent.com> - 1:29.1-4
- fix CVE-2024-30202/30203/30204.

* Tue Mar 19 2024 Rebuild Robot <rebot@opencloudos.org> - 1:29.1-3
- Rebuilt for jansson

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1:29.1-2
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Sep 01 2023 cunshunxia <cunshunxia@tencent.com> - 1:29.1-1
- upgrade to 29.1

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1:27.2-5
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1:27.2-4
- Rebuilt for OpenCloudOS Stream 23

* Thu Feb 23 2023 Zhao Zhen <jeremiazhao@tencent.com> - 1:27.2-3
- updated reversion

* Tue Sep 13 2022 Zhao Zhen <jeremiazhao@tencent.com> - 1:27.2-2
- only build nonX version

* Mon Apr 18 2022 Zhao Zhen <jeremiazhao@tencent.com> - 1:27.2-1
- initial
